@@ -24,11 +24,12 @@ from .plot_utils import _transform_basis, _get_nrows_and_ncols, _get_marker_size
 
 def scatter(
     data: Union[MultimodalData, UnimodalData, anndata.AnnData],
-    attrs: Union[str, List[str]],
+    attrs: Union[str, List[str]] = None,
     basis: Optional[str] = "umap",
     matkey: Optional[str] = None,
     restrictions: Optional[Union[str, List[str]]] = None,
     show_background: Optional[bool] = False,
+    fix_corners: Optional[bool] = True,
     alpha: Optional[Union[float, List[float]]] = 1.0,
     legend_loc: Optional[Union[str, List[str]]] = "right margin",
     legend_ncol: Optional[str] = None,
@@ -53,8 +54,8 @@ def scatter(
     ----------
     data: ``pegasusio.MultimodalData``
        Use current selected modality in data.
-    attrs: ``str`` or ``List[str]``
-        Color scatter plots by attrs. Each attribute in attrs should be one key in data.obs or data.var_names (e.g. one gene). If one attribute is categorical, a palette will be used to color each category separately. Otherwise, a color map will be used.
+    attrs: ``str`` or ``List[str]``, default: None
+        Color scatter plots by attrs. Each attribute in attrs can be one key in data.obs, data.var_names (e.g. one gene) or data.obsm (attribute has the format of 'obsm_key@component', like 'X_pca@0'). If one attribute is categorical, a palette will be used to color each category separately. Otherwise, a color map will be used. If no attributes are provided, plot the basis for all data.
     basis: ``str``, optional, default: ``umap``
         Basis to be used to generate scatter plots. Can be either 'umap', 'tsne', 'fitsne', 'fle', 'net_tsne', 'net_fitsne', 'net_umap' or 'net_fle'.
     matkey: ``str``, optional, default: None
@@ -63,6 +64,8 @@ def scatter(
         A list of restrictions to subset data for plotting. There are two types of restrictions: global restriction and attribute-specific restriction. Global restriction appiles to all attributes in ``attrs`` and takes the format of 'key:value,value...', or 'key:~value,value...'. This restriction selects cells with the ``data.obs[key]`` values belong to 'value,value...' (or not belong to if '~' shows). Attribute-specific restriction takes the format of 'attr:key:value,value...', or 'attr:key:~value,value...'. It only applies to one attribute 'attr'. If 'attr' and 'key' are the same, one can use '.' to replace 'key' (e.g. ``cluster_labels:.:value1,value2``).
     show_background: ``bool``, optional, default: False
         Only applicable if `restrictions` is set. By default, only data points selected are shown. If show_background is True, data points that are not selected will also be shown.
+    fix_corners: ``bool``, optional, default: True
+        If True, fix the corners of the plots as defined using all data points.
     alpha: ``float`` or ``List[float]``, optional, default: ``1.0``
         Alpha value for blending, from 0.0 (transparent) to 1.0 (opaque). If this is a list, the length must match attrs, which means we set a separate alpha value for each attribute.
     legend_loc: ``str`` or ``List[str]``, optional, default: ``right margin``
@@ -107,7 +110,11 @@ def scatter(
     >>> pg.scatter(data, attrs=['louvain_labels', 'Channel'], basis='fitsne')
     >>> pg.scatter(data, attrs=['CD14', 'TRAC'], basis='umap')
     """
-    if not is_list_like(attrs):
+    if attrs is None:
+        attrs = ['_all'] # default, plot all points
+        if palettes is None:
+            palettes = '_all:slategrey'
+    elif not is_list_like(attrs):
         attrs = [attrs]
     nattrs = len(attrs)
 
@@ -126,7 +133,7 @@ def scatter(
 
 
     basis = _transform_basis(basis)
-    marker_size = _get_marker_size(x.size)
+    global_marker_size = _get_marker_size(x.size)
     nrows, ncols = _get_nrows_and_ncols(nattrs, nrows, ncols)
     fig, axes = _get_subplot_layouts(nrows=nrows, ncols=ncols, panel_size=panel_size, dpi=dpi, left=left, bottom=bottom, wspace=wspace, hspace=hspace, squeeze=False)
 
@@ -136,7 +143,7 @@ def scatter(
 
     if not is_list_like(legend_loc):
         legend_loc = [legend_loc] * nattrs
-    legend_fontsize = [5 if x == 'on data' else 10 for x in legend_loc]
+    legend_fontsize = [5 if x == "on data" else 10 for x in legend_loc]
 
     palettes = DictWithDefault(palettes)
     cmaps = DictWithDefault(cmaps)
@@ -154,23 +161,31 @@ def scatter(
                 pos = i * ncols + j
                 attr = attrs[pos]
 
-                if attr in data.obs:
+                if attr == '_all': # if default
+                    values = pd.Categorical.from_codes(np.zeros(data.shape[0], dtype=int), categories=['cell'])
+                elif attr in data.obs:
                     values = data.obs[attr].values
-                else:
-                    try:
-                        loc = data.var_names.get_loc(attr)
-                    except KeyError:
-                        raise KeyError(f"{attr} is neither in data.obs nor data.var_names!")
+                elif attr in data.var_names:
+                    loc = data.var_names.get_loc(attr)
                     values = slicing(data.X, col = loc)
+                else:
+                    obsm_key, sep, component = attr.partition("@")
+                    if (sep != "@") or (obsm_key not in data.obsm) or (not component.isdigit()): 
+                        raise KeyError(f"{attr} is not in data.obs, data.var_names or data.obsm!")
+                    values = data.obsm[obsm_key][:, int(component)]
 
                 selected = restr_obj.get_satisfied(data, attr)
+                marker_size = global_marker_size
+                if (not fix_corners) and (is_numeric_dtype(values) or (not show_background)):
+                    marker_size = _get_marker_size(selected.sum())
 
                 if is_numeric_dtype(values):
                     cmap = cmaps.get(attr, squeeze = True)
                     if cmap is None:
                         raise KeyError(f"Please set colormap for attribute {attr} or set a default colormap!")
 
-                    _plot_corners(ax, corners, marker_size)
+                    if fix_corners:
+                        _plot_corners(ax, corners, marker_size)
 
                     img = ax.scatter(
                         x[selected],
@@ -222,26 +237,29 @@ def scatter(
                                     **scatter_kwargs,
                                 )
                             else:
-                                _plot_corners(ax, corners, marker_size)
+                                if fix_corners:
+                                    _plot_corners(ax, corners, marker_size)
 
-                    if legend_loc[pos] == "right margin":
-                        legend = ax.legend(
-                            loc="center left",
-                            bbox_to_anchor=(1, 0.5),
-                            frameon=False,
-                            fontsize=legend_fontsize[pos],
-                            ncol=_get_legend_ncol(label_size, legend_ncol),
-                        )
-                        for handle in legend.legendHandles:
-                            handle.set_sizes([300.0])
-                    elif legend_loc[pos] == "on data":
-                        texts = []
-                        for px, py, txt in text_list:
-                            texts.append(ax.text(px, py, txt, fontsize=legend_fontsize[pos], fontweight = "bold", ha = "center", va = "center"))
-                        # from adjustText import adjust_text
-                        # adjust_text(texts, arrowprops=dict(arrowstyle='-', color='k', lw=0.5))
+                    if attr != '_all':
+                        if legend_loc[pos] == "right margin":
+                            legend = ax.legend(
+                                loc="center left",
+                                bbox_to_anchor=(1, 0.5),
+                                frameon=False,
+                                fontsize=legend_fontsize[pos],
+                                ncol=_get_legend_ncol(label_size, legend_ncol),
+                            )
+                            for handle in legend.legendHandles:
+                                handle.set_sizes([300.0])
+                        elif legend_loc[pos] == "on data":
+                            texts = []
+                            for px, py, txt in text_list:
+                                texts.append(ax.text(px, py, txt, fontsize=legend_fontsize[pos], fontweight = "bold", ha = "center", va = "center"))
+                            # from adjustText import adjust_text
+                            # adjust_text(texts, arrowprops=dict(arrowstyle='-', color='k', lw=0.5))
 
-                ax.set_title(attr)
+                if attr != '_all':
+                    ax.set_title(attr)
             else:
                 ax.set_frame_on(False)
 
@@ -292,7 +310,7 @@ def scatter_groups(
     data: ``pegasusio.MultimodalData``
        Use current selected modality in data.
     attr: ``str``
-        Color scatter plots by attribute 'attr'. This attribute should be one key in data.obs or data.var_names (e.g. one gene). If it is categorical, a palette will be used to color each category separately. Otherwise, a color map will be used.
+        Color scatter plots by attribute 'attr'. This attribute should be one key in data.obs, data.var_names (e.g. one gene) or data.obsm (attribute has the format of 'obsm_key@component', like 'X_pca@0'). If it is categorical, a palette will be used to color each category separately. Otherwise, a color map will be used.
     groupby: ``str``
         Generate separate scatter plots of 'attr' for data points in each category in 'groupby', which should be a key in data.obs representing one categorical variable.
     basis: ``str``, optional, default: ``umap``
@@ -366,12 +384,14 @@ def scatter_groups(
 
     if attr in data.obs:
         values = data.obs[attr].values
-    else:
-        try:
-            loc = data.var_names.get_loc(attr)
-        except KeyError:
-            raise KeyError(f"{attr} is neither in data.obs nor data.var_names!")
+    elif attr in data.var_names:
+        loc = data.var_names.get_loc(attr)
         values = slicing(data.X, col = loc)
+    else:
+        obsm_key, sep, component = attr.partition("@")
+        if (sep != "@") or (obsm_key not in data.obsm) or (not component.isdigit()): 
+            raise KeyError(f"{attr} is not in data.obs, data.var_names or data.obsm!")
+        values = data.obsm[obsm_key][:, int(component)]
 
     is_cat = is_categorical_dtype(values)
     if (not is_cat) and (not is_numeric_dtype(values)):
@@ -509,6 +529,8 @@ def compo_plot(
     groupby_label: Optional[str] = None,
     sort_function: Union[Callable[[List[str]], List[str]], str] = 'natsorted',
     panel_size: Optional[Tuple[float, float]] = (6, 4),
+    palette: Optional[List[str]] = None,
+    color_unused: bool = False,
     left: Optional[float] = 0.15,
     bottom: Optional[float] = 0.15,
     wspace: Optional[float] = 0.3,
@@ -524,31 +546,33 @@ def compo_plot(
     Parameters
     ----------
 
-    data : `AnnData` or `UnimodalData` or `MultimodalData` object
+    data : ``AnnData`` or ``UnimodalData`` or ``MultimodalData`` object
         Single cell expression data.
-    groupby : `str`
+    groupby : ``str``
         A categorical variable in data.obs that is used to categorize the cells, e.g. cell type.
-    condition: `str`
-        A categorical variable in data.obs that is used to calculate frequency within each category defined by 'groupby', e.g. donor.
-    style: `str`, optional (default: `frequency`)
-        Composition plot style. Can be either `frequency`, or 'normalized'. Within each cluster, the `frequency` style show the percentage of cells from each 'condition' within each category in 'groupby' (stacked), the `normalized` style shows for each category in 'groupby' the percentage of cells that are also in each 'condition' over all cells that are in the same 'condition' (not stacked).
+    condition: ``str``
+        A categorical variable in data.obs that is used to calculate frequency within each category defined by ``groupby``, e.g. donor.
+    style: ``str``, optional (default: ``frequency``)
+        Composition plot style. Can be either ``frequency``, or ``normalized``. Within each cluster, the ``frequency`` style show the percentage of cells from each ``condition`` within each category in ``groupby`` (stacked), the ``normalized`` style shows for each category in ``groupby`` the percentage of cells that are also in each ``condition`` over all cells that are in the same ``condition`` (not stacked).
     restrictions: ``str`` or ``List[str]``, optional, default: None
         A list of restrictions to subset data for plotting. Each restriction takes the format of 'key:value,value...', or 'key:~value,value...'. This restriction selects cells with the ``data.obs[key]`` values belong to 'value,value...' (or not belong to if '~' shows).
     switch_axes: ``bool``, optional, default: ``False``
         By default, X axis is for groupby, and Y axis for frequencies with respect to condition. If this parameter is ``True``, switch the axes.
-    groupby_label: `str`, optional (default None)
-        Label for the axis displaying 'groupby' categories. If None, use 'groupby'.
+    groupby_label: ``str``, optional (default ``None``)
+        Label for the axis displaying ``groupby`` categories. If ``None``, use ``groupby``.
     sort_function: ``Union[Callable[List[str], List[str]], str]``, optional, default: ``natsorted``
         Function used for sorting both groupby and condition labels. If ``natsorted``, apply natsorted function to sort by natural order. If ``None``, don't sort. Otherwise, a callable function will be applied to the labels for sorting.
-    panel_size: `tuple`, optional (default: `(6, 4)`)
+    panel_size: ``tuple``, optional (default: ``(6, 4)``)
         The plot size (width, height) in inches.
-    left: `float`, optional (default: `0.15`)
+    palette: ``List[str]``, optional (default: ``None``)
+        Used for setting colors for categories in ``condition``. Within the list, each string is the color for one category.
+    left: ``float``, optional (default: ``0.15``)
         This parameter sets the figure's left margin as a fraction of panel's width (left * panel_size[0]).
-    bottom: `float`, optional (default: `0.15`)
+    bottom: ``float``, optional (default: ``0.15``)
         This parameter sets the figure's bottom margin as a fraction of panel's height (bottom * panel_size[1]).
-    wspace: `float`, optional (default: `0.3`)
+    wspace: ``float``, optional (default: ``0.3``)
         This parameter sets the width between panels and also the figure's right margin as a fraction of panel's width (wspace * panel_size[0]).
-    hspace: `float`, optional (defualt: `0.15`)
+    hspace: ``float``, optional (defualt: ``0.15``)
         This parameter sets the height between panels and also the figure's top margin as a fraction of panel's height (hspace * panel_size[1]).
     return_fig: ``bool``, optional, default: ``False``
         Return a ``Figure`` object if ``True``; return ``None`` otherwise.
@@ -558,7 +582,7 @@ def compo_plot(
     Returns
     -------
 
-    `Figure` object
+    ``Figure`` object
         A ``matplotlib.figure.Figure`` object containing the dot plot if ``return_fig == True``
 
     Examples
@@ -593,11 +617,25 @@ def compo_plot(
         assert style == "normalized"
         df = df.div(df.sum(axis=0), axis=1) * 100.0
 
+    if color_unused:
+        if palette is None:
+            color_list = _get_palette(data.obs[condition].cat.categories.size)
+        else:
+            assert len(palette) >= data.obs[condition].cat.categories.size, "The palette provided has fewer colors than needed!"
+            color_idx = df.columns.map(data.obs[condition].cat.categories.get_loc)
+            color_list = palette[color_idx]
+    else:
+        if palette is None:
+            color_list = _get_palette(df.shape[1])
+        else:
+            assert len(palette) >= df.shape[1], "The palette provided has fewer colors than needed!"
+            color_list = palette[0:df.shape[1]]
+
     df.plot(
         kind = "bar" if not switch_axes else "barh",
         stacked = style == "frequency",
         legend = False,
-        color = _get_palette(df.shape[1]),
+        color = color_list,
         ax = ax,
     )
 
@@ -622,9 +660,11 @@ def violin(
     groupby: str,
     hue: Optional[str] = None,
     matkey: Optional[str] = None,
-    stripplot: bool = False,
-    scale: str = 'width',
+    stripplot: Optional[bool] = False,
+    inner: Optional[str] = None,
+    scale: Optional[str] = 'width',
     panel_size: Optional[Tuple[float, float]] = (8, 0.5),
+    palette: Optional[List[str]] = None,
     left: Optional[float] = 0.15,
     bottom: Optional[float] = 0.15,
     wspace: Optional[float] = 0.1,
@@ -652,16 +692,21 @@ def violin(
         If matkey is set, select matrix with matkey as keyword in the current modality. Only works for MultimodalData or UnimodalData objects.
     stripplot: ``bool``, optional, default: ``False``
         Attach a stripplot to the violinplot or not. This option will be automatically turn off if 'hue' is set.
+    inner: ``str``, optional, default: ``None``
+        Representation of the datapoints in the violin interior:
+            - If ``box``, draw a miniature boxplot.
+            - If ``quartiles``, draw the quartiles of the distribution.
+            - If ``point`` or ``stick``, show each underlying datapoint.
+            - If ``None``, will draw unadorned violins.
     scale: ``str``, optional, default: ``width``
         The method used to scale the width of each violin:
             - If ``width``, each violin will have the same width.
             - If ``area``, each violin will have the same area.
             - If ``count``, the width of the violins will be scaled by the number of observations in that bin.
-    jitter: ``float`` or ``bool``, optional, default: ``False``
-        Amount of jitter (only along the categorical axis) to apply to stripplot. This is used only when ``stripplot`` is set to ``True``.
-        This can be useful when you have many points and they overlap, so that it is easier to see the distribution. You can specify the amount of jitter (half the width of the uniform random variable support), or just use ``True`` for a good default.
     panel_size: ``Tuple[float, float]``, optional, default: ``(8, 0.5)``
         The size (width, height) in inches of each violin panel.
+    palette: ``List[str]``, optional (default: ``None``)
+        Used for setting colors for categories in ``groupby``. Within the list, each string is the color for one category.
     left: ``float``, optional, default: ``0.15``
         This parameter sets the figure's left margin as a fraction of panel's width (left * panel_size[0]).
     bottom: ``float``, optional, default: ``0.15``
@@ -726,7 +771,7 @@ def violin(
         ax = axes[i, 0]
         if stripplot:
             sns.stripplot(x="label", y=attrs[i], hue = hue, data=df, ax=ax, size=1, color="k", jitter=True)
-        sns.violinplot(x="label", y=attrs[i], hue = hue, data=df, inner=None, linewidth=1, ax=ax, cut=0, scale=scale, split=True, **kwargs)
+        sns.violinplot(x="label", y=attrs[i], hue = hue, data=df, inner=inner, linewidth=1, ax=ax, cut=0, scale=scale, split=True, palette=palette, **kwargs)
         ax.grid(False)
 
         if hue is not None:
@@ -856,7 +901,7 @@ def heatmap(
     if not is_categorical_dtype(clusters):
         clusters = pd.Categorical(clusters)
     else:
-        clusters.remove_unused_categories(inplace = True)
+        clusters = clusters.remove_unused_categories()
     df_list = [pd.DataFrame({'cluster_name': clusters})]
 
     if len(obs_keys) > 0:
@@ -900,7 +945,7 @@ def heatmap(
         )
         cg.ax_heatmap.set_ylabel("")
         if attrs_labelsize is not None:
-            cg.ax_heatmap.tick_params(axis='x', labelsize=attrs_labelsize)
+            cg.ax_heatmap.tick_params(axis='x', labelsize=attrs_labelsize, labelrotation=75)
     else:
         cg = sns.clustermap(
             data=df.T,
@@ -1693,7 +1738,7 @@ def rank_plot(
 
     Examples
     --------
-    >>> fig = pg.rank_plot(data, show = False, dpi = 500)
+    >>> fig = pg.rank_plot(data, dpi = 500)
     """
     fig, ax = _get_subplot_layouts(panel_size=panel_size, dpi=dpi) # default nrows = 1 & ncols = 1
 
@@ -1838,3 +1883,72 @@ def ridgeplot(
     sns.reset_orig()
 
     return g.fig if return_fig else None
+
+
+def wordcloud(
+    data: Union[MultimodalData, UnimodalData, anndata.AnnData],
+    factor: int,
+    max_words: Optional[int] = 20,
+    random_state: Optional[int] = 0,
+    colormap: Optional[str] = "hsv",
+    width: Optional[int] = 800,
+    height: Optional[int] = 400,
+    panel_size: Optional[Tuple[float, float]] = (6, 4),
+    return_fig: Optional[bool] = False,
+    dpi: Optional[float] = 300.0,
+    **kwargs,
+) -> Union[plt.Figure, None]:
+    """Generate one word cloud image for factor (starts from 0) in data.uns['W']. 
+
+    Parameters
+    ----------
+
+    data : ``AnnData`` or ``UnimodalData`` or ``MultimodalData`` object
+        The main data object.
+    factor: ``int``
+        Which factor to plot. factor starts from 0.
+    max_words: ``int``, optional, default: 20
+        Maximum number of genes to show in the image.
+    random_state: ``int``, optional, default: 0
+        Random seed passing to WordCloud function.
+    colormap: ``str``, optional, default: ``hsv``
+        Color map for plotting words.
+    width: ``int``, optional, default: 800
+        Canvas width.
+    height: ``int``, optional, default: 400
+        Canvas height.
+    panel_size: ``tuple``, optional, default: `(6, 4)`
+        The plot size (width, height) in inches.
+    return_fig: ``bool``, optional, default: ``False``
+        Return a ``Figure`` object if ``True``; return ``None`` otherwise.
+    dpi: ``float``, optional, default: ``300.0``
+        The resolution in dots per inch.
+
+    Returns
+    -------
+
+    `Figure` object
+        A ``matplotlib.figure.Figure`` object containing the dot plot if ``return_fig == True``
+
+    Examples
+    --------
+    >>> fig = pg.wordcloud(data, factor=0)
+    """
+    fig, ax = _get_subplot_layouts(panel_size=panel_size, dpi=dpi) # default nrows = 1 & ncols = 1
+
+    assert 'W' in data.uns
+    hvg = data.var_names[data.var['highly_variable_features']]
+    word_dict = {}
+    for i in range(hvg.size):
+        word_dict[hvg[i]] = data.uns['W'][i, factor]
+
+    from wordcloud import WordCloud
+    wc = WordCloud(background_color="white", max_words=max_words, random_state=random_state, colormap=colormap, width=width, height=height)
+    wc.generate_from_frequencies(word_dict)
+    
+    ax.imshow(wc)
+    ax.axis('off')
+
+    return fig if return_fig else None
+
+
