@@ -33,7 +33,7 @@ from .nearest_neighbors import (
 from .graph_operations import construct_graph
 from .diffusion_map import diffmap
 from .pseudotime import calc_pseudotime, infer_path
-from .clustering import jump_method, louvain, leiden, spectral_louvain, spectral_leiden, cluster
+from .clustering import jump_method, louvain, leiden, spectral_louvain, spectral_leiden, cluster, split_one_cluster
 from .net_regressor import net_train_and_predict
 from .visualization import (
     tsne,
@@ -47,3 +47,4 @@ from .gradient_boosting import find_markers, run_find_markers
 from .subcluster_utils import get_anndata_for_subclustering
 from .signature_score import calc_signature_score
 from .doublet_detection import infer_doublets, mark_doublets
+from .nmf import nmf, integrative_nmf
