@@ -150,9 +150,14 @@ def _de_test(
         dftq = pd.DataFrame(qvals, index = gene_names, columns = [f"{x}:t_qval" for x in cluster_labels.categories])
         df_list.extend([dft, dftp, dftq])
 
+        if verbose:
+            end = time.perf_counter()
+            logger.info(f"Welch's t-test is finished. Time spent = {end - start:.4f}s.")
+            start = end
+
     if fisher:
         from pegasus.cylib.cfisher import fisher_exact
-        a_true, a_false, b_true, b_false = results[2]
+        a_true, a_false, b_true, b_false = results[1 if not t else 2]
 
         oddsratios = np.zeros((ngene, n1arr.size), dtype = np.float32)
         pvals = np.ones((ngene, n1arr.size), dtype = np.float32)
@@ -253,7 +258,7 @@ def _perform_de_cond(
             df_list.extend([dft, dftp, dftq])
 
         if fisher:
-            a_true, a_false, b_true, b_false = results[2]
+            a_true, a_false, b_true, b_false = results[1 if not t else 2]
 
             from pegasus.cylib.cfisher import fisher_exact
 
@@ -503,11 +508,11 @@ def _assemble_df(res_dict: dict, rec_array: np.ndarray, prefix: str, col_names:
     df.columns = col_names
 
     idx = df["mwu_qval"] <= alpha
-    idx_up = idx & (df["log2FC"].values > 0)
+    idx_up = idx & (df["auroc"].values > 0.5)
     df_up = df.loc[idx_up].sort_values(by="auroc", ascending=False, inplace=False)
     res_dict["up"] = pd.DataFrame(df_up if head is None else df_up.iloc[0:head])
 
-    idx_down = idx & (df["log2FC"].values < 0)
+    idx_down = idx & (df["auroc"].values < 0.5)
     df_down = df.loc[idx_down].sort_values(by="auroc", ascending=True, inplace=False)
     res_dict["down"] = pd.DataFrame(df_down if head is None else df_down.iloc[0:head])
 
@@ -536,7 +541,7 @@ def markers(
         Keyword of DE result stored in ``data.varm``.
 
     alpha: ``float``, optional, default: ``0.05``
-        q-value threshold for getting significant DE genes. Only those with q-value of any test no less than ``alpha`` are significant, and thus considered as DE genes.
+        q-value threshold for getting significant DE genes. Only those with q-value of MWU test no less than ``alpha`` are significant, and thus considered as DE genes.
 
     Returns
     -------
@@ -544,6 +549,8 @@ def markers(
         A Python dictionary containing markers.
         If DE is performed between clusters, the structure is ``dict[cluster_id]['up' or 'down'][dataframe]``.
         If DE is performed between conditions within each cluster, the structure is ``dict[cluster_id][condition_id]['up' or 'down'][dataframe]``.
+        'up' refers to up-regulated genes, which should have 'auroc' > 0.5.
+        'down' refers to down-regulated genes, which should have 'auroc' < 0.5.
 
     Examples
     --------
@@ -689,7 +696,7 @@ def write_results_to_excel(
     ) -> pd.DataFrame:
         """ Round related float columns to ndigits decimal points.
         """
-        df = pd.DataFrame(df_orig)
+        df = pd.DataFrame(df_orig, copy = True) # copy must be true, otherwise the passed df_orig will be modified.
 
         cols = []
         for name in df.columns:
