@@ -49,8 +49,8 @@ def _find_local_maxima(y: List[float], frac: float = 0.25, merge_peak_frac: floa
                 maxima_by_x.append(i)
             else:
                 filtered_maxima.append(i)
-    maxima_by_x = np.array(maxima_by_x)
-    filtered_maxima = np.array(filtered_maxima)
+    maxima_by_x = np.array(maxima_by_x, dtype=int)
+    filtered_maxima = np.array(filtered_maxima, dtype=int)
     n_max = maxima_by_x.size
 
     curr_peak = 0
@@ -65,19 +65,14 @@ def _find_local_maxima(y: List[float], frac: float = 0.25, merge_peak_frac: floa
             if y[maxima_by_x[i + 1]] > y[maxima_by_x[curr_peak]]:
                 curr_peak = i + 1
     merged_peaks.append(maxima_by_x[curr_peak])
-    merged_peaks = np.array(merged_peaks)
+    merged_peaks = np.array(merged_peaks, dtype=int)
     maxima = merged_peaks[np.argsort(y[merged_peaks])[::-1]]
 
     return maxima, maxima_by_x, filtered_maxima
 
-def _locate_cutoff_among_peaks(y: List[float], maxima: List[float]) -> int:
-    """ Find two peaks with largest gap.
-    """
-    best_gap = 0
+def _locate_cutoff_among_peaks_with_guide(x: List[float], y: List[float], maxima: List[float], sim_scores_log: List[float], d_neo: float) -> int:
+    best_delta = 1e100
     best_pos = -1
-
-    pos_12 = gap_12 = -1
-
     for i in range(1, maxima.size):
         if maxima[0] < maxima[i]:
             start = maxima[0]
@@ -86,23 +81,18 @@ def _locate_cutoff_among_peaks(y: List[float], maxima: List[float]) -> int:
             start = maxima[i]
             end = maxima[0]
         pos = y[start+1:end].argmin() + (start+1)
-        gap = y[maxima[i]] - y[pos]
-
-        if i == 1:
-            pos_12 = pos
-            gap_12 = y[maxima[0]] - y[pos]
-
-        if best_gap < gap:
-            best_gap = gap
+        d_prac_neo = (sim_scores_log > x[pos]).sum() / sim_scores_log.size
+        delta = abs(d_prac_neo - d_neo)
+        if best_delta > delta:
+            best_delta = delta
             best_pos = pos
+    return best_pos
 
-    return best_pos if best_gap > gap_12 else pos_12
-
-def _find_pos_curv(curv, start, dir, thre = 0.06):
+def _find_pos_curv(curv, start, dir, err_bound = 0.05):
     RANGE = range(start, curv.size) if dir == '+' else range(start, 0, -1)
     assert (RANGE.stop - RANGE.start) * RANGE.step > 0
     for pos in RANGE:
-        if curv[pos] > thre:
+        if curv[pos] > err_bound:
             break
     return pos
 
@@ -116,50 +106,48 @@ def _find_curv_minima_at_peak(curv, peak_pos):
         end += 1
     return curv[start:end].min()
 
-def _find_curv_local_minima(curv, peak_curv_value, filtered_maxima, start, dir, rel_thre = 0.45, minima_dir_thre = -0.25):
-    """ Find a negative curvature value that is a local minima or a filtered local maxima with respect to density value.
-        dir represents the direction of search, choosing from '+' or '-'.
+def _find_curv_local_minima(curv, peak_curv_value, filtered_maxima, start, rel_thre = 0.45, minima_dir_thre = -0.25):
+    """ Find a negative curvature value that is a local minima or a filtered local maxima with respect to density value at the right hand side of start.
         Beside being a local minima, the value must also satisfy the rel_thre requirement.
         rel_thre requires that the curvature value must smaller than rel_thre fraction of the max of minimal curvature value of the peak and the minimal curvature value since start at direction dir.
     """
-    if dir == '+':
-        pos_from = max(start, 2)
-        pos_to = curv.size - 2
-        tmp_arr = filtered_maxima[filtered_maxima > start]
-        if tmp_arr.size > 0:
-            lmax = tmp_arr.min()
-            pos_to = _find_pos_curv(curv, lmax-1, '-') + 1
-        assert pos_from < pos_to
-        minima_with_dir = curv[pos_from:pos_to].min()
-        if minima_with_dir >= minima_dir_thre:
-            # No other local minima
-            return pos_to # return right end
-        thre = min(max(peak_curv_value, minima_with_dir) * rel_thre, minima_dir_thre)
-        assert thre < 0.0
-        for pos in range(pos_from, pos_to):
-            if curv[pos] < thre and curv[pos - 1] > curv[pos] and curv[pos] < curv[pos + 1]:
-                return pos
-        assert False
-    else:
-        assert dir == '-'
-        pos_from = min(start, curv.size - 3)
-        pos_to = 1
-        tmp_arr = filtered_maxima[filtered_maxima < start]
-        if tmp_arr.size > 0:
-            lmax = tmp_arr.max()
-            pos_to = _find_pos_curv(curv, lmax+1, '+') - 1
-        assert pos_from > pos_to
-        minima_with_dir = curv[pos_to+1:pos_from+1].min()
-        if minima_with_dir >= minima_dir_thre:
-            return pos_to
-        thre = min(max(peak_curv_value, minima_with_dir) * rel_thre, minima_dir_thre)
-        assert thre < 0.0
-        for pos in range(pos_from, pos_to, -1):
-            if curv[pos] < thre and curv[pos - 1] > curv[pos] and curv[pos] < curv[pos + 1]:
-                return pos
-        assert False
+    pos_from = max(start, 2)
+    pos_to = curv.size - 2
+    tmp_arr = filtered_maxima[filtered_maxima > start]
+    if tmp_arr.size > 0:
+        lmax = tmp_arr.min()
+        pos_to = _find_pos_curv(curv, lmax-1, '-') + 1
+    assert pos_from < pos_to
+    minima_with_dir = curv[pos_from:pos_to].min()
+    if minima_with_dir >= minima_dir_thre:
+        # No other local minima
+        return pos_to # return right end
+    thre = min(max(peak_curv_value, minima_with_dir) * rel_thre, minima_dir_thre)
+    assert thre < 0.0
+    for pos in range(pos_from, pos_to):
+        if curv[pos] < thre and curv[pos - 1] > curv[pos] and curv[pos] < curv[pos + 1]:
+            return pos
+    assert False
 
-def _plot_hist(obs_scores, sim_scores, threshold, sim_x, sim_y, curv, nbin = 100, fig_size = (8,6), dpi = 300):
+def _find_cutoff_left_side(peak_pos: int, x: List[float], curv: List[float], x_theory: float) -> int:
+    # Peak represents a doublet peak and thus we need to find a cutoff at the left side
+    end = _find_pos_curv(curv, peak_pos-1, '-')
+    start = end
+    while start > 2 and x[start] >= x_theory:
+        start -= 1
+    while start > 2 and not (curv[start - 1] > curv[start] and curv[start] < curv[start + 1]):
+        start -= 1
+    return start + curv[start:end+1].argmax()
+
+def _find_cutoff_right_side(peak_pos: int, curv: List[float], filtered_maxima: List[int]) -> int:
+    # Peak represents embedded doublets, find a cutoff at the right side
+    peak_curv_value = _find_curv_minima_at_peak(curv, peak_pos)
+    start = _find_pos_curv(curv, peak_pos+1, '+')
+    end = _find_pos_curv(curv, _find_curv_local_minima(curv, peak_curv_value, filtered_maxima, start+1)-1, '-')
+    assert start <= end
+    return curv[start:end+1].argmax() + start
+
+def _plot_hist(obs_scores, sim_scores, threshold, threshold_theory, sim_x, sim_y, curv, nbin = 100, fig_size = (8,6), dpi = 300, threshold_auto = None):
     """ Plot histogram of doublet scores for observed cells and simulated doublets
         (A) top left: histogram of observed cells;
         (B) top right: histogram of simulated doublets;
@@ -174,6 +162,9 @@ def _plot_hist(obs_scores, sim_scores, threshold, sim_x, sim_y, curv, nbin = 100
     ax.hist(obs_scores, x, color="gray", linewidth=0, density=True)
     ax.set_yscale("log")
     ax.axvline(x = threshold, ls = "--", c = "k", linewidth=1)
+    ax.axvline(x = threshold_theory, ls = "--", c = "r", linewidth=1)
+    if threshold_auto is not None:
+        ax.axvline(x = threshold_auto, ls = "--", c = "g", linewidth=1)
     ax.set_title('Observed cells')
     ax.set_xlabel('Doublet score')
     ax.set_ylabel('Density')
@@ -182,24 +173,20 @@ def _plot_hist(obs_scores, sim_scores, threshold, sim_x, sim_y, curv, nbin = 100
     ax.hist(sim_scores, x, color="gray", linewidth=0, density=True)
     ax.set_yscale("log")
     ax.axvline(x = threshold, ls = "--", c = "k", linewidth=1)
+    ax.axvline(x = threshold_theory, ls = "--", c = "r", linewidth=1)
+    if threshold_auto is not None:
+        ax.axvline(x = threshold_auto, ls = "--", c = "g", linewidth=1)
     ax.set_title('Simulated doublets')
     ax.set_xlabel('Doublet score')
     ax.set_ylabel('Density')
 
-    # ax = axes[1, 0]
-    # from scipy.stats import gaussian_kde
-    # kde = gaussian_kde(sim_scores)
-    # y = kde(x)
-    # ax.plot(x, y, '-', c='k', lw = 1)
-    # ax.set_ylim(bottom = 0.0)
-    # ax.set_title('KDE of simulated doublets')
-    # ax.set_xlabel('Doublet score')
-    # ax.set_ylabel('Density')
-
     ax = axes[1, 0]
     ax.plot(sim_x, sim_y, '-', c='k', lw = 1)
     ax.set_ylim(bottom = 0.0)
     ax.axvline(x = np.log(threshold), ls = "--", c="k", lw=1)
+    ax.axvline(x = np.log(threshold_theory), ls = "--", c="r", lw=1)
+    if threshold_auto is not None:
+        ax.axvline(x = np.log(threshold_auto), ls = "--", c = "g", linewidth=1)
     ax.set_title('KDE of simulated doublets')
     ax.set_xlabel('Log doublet score')
     ax.set_ylabel('Density')
@@ -207,6 +194,9 @@ def _plot_hist(obs_scores, sim_scores, threshold, sim_x, sim_y, curv, nbin = 100
     ax = axes[1, 1]
     ax.plot(sim_x, curv, '-', c='k', lw = 1)
     ax.axvline(x = np.log(threshold), ls = "--", c="k", lw=1)
+    ax.axvline(x = np.log(threshold_theory), ls = "--", c="r", lw=1)
+    if threshold_auto is not None:
+        ax.axvline(x = np.log(threshold_auto), ls = "--", c = "g", linewidth=1)
     ax.set_title('Curvature of simulated doublets')
     ax.set_xlabel('Log doublet score')
     ax.set_ylabel('Curvature')
@@ -237,7 +227,8 @@ def _run_scrublet(
     k: Optional[int] = None,
     n_jobs: Optional[int] = -1,
     random_state: Optional[int] = 0,
-    plot_hist: Optional[bool] = True
+    plot_hist: Optional[bool] = True,
+    manual_correction: Optional[str] = None,
 ) -> Union[None, Figure]:
     """Calculate doublet scores using Scrublet-like [Wolock18]_ strategy for the current data.X; determine a right threshold based on the KDE curve.
        This function should be called after highly_variable_gene selection.
@@ -271,6 +262,9 @@ def _run_scrublet(
     plot_hist: ``bool``, optional, default: ``True``
         If True, plot diagnostic histograms. Each sample would have a figure consisting of 4 panels showing histograms of doublet scores for observed cells (panel 1, density in log scale), simulated doublets (panel 2, density in log scale), KDE plot (panel 3) and signed curvature plot (panel 4) of log doublet scores for simulated doublets.
 
+    manual_correction: ``str``, optional, default: ``None``
+        If present, use human guide provided in manual_correction to select threshold. Currently only support manual_correction='peak', which means cut at the center of the peak.
+
     Returns
     --------
     ``None`` or a ``matplotlib Figure object`` if
@@ -289,6 +283,7 @@ def _run_scrublet(
     from pegasus.tools import calculate_nearest_neighbors, simulate_doublets
     from sklearn.decomposition import PCA
     from scipy.stats import gaussian_kde
+    from sklearn.cluster import KMeans
 
     if "highly_variable_features" not in data.var:
         raise ValueError("_run_scrublet must be run after highly_variable_features is called!")
@@ -323,6 +318,14 @@ def _run_scrublet(
     with threadpool_limits(limits = n_jobs):
         obs_pca = pca.fit_transform(obsX.astype(np.float64)) # float64 for reproducibility
         obs_pca = np.ascontiguousarray(obs_pca, dtype=np.float32)
+        kmeans = KMeans(n_clusters = 5, random_state = random_state).fit(obs_pca)
+
+    # calculate in simulated distribution, expected percentage of embedded doublets
+    data.obs["dbl_kmeans_"] = pd.Categorical(kmeans.labels_)
+    _, freqs = np.unique(kmeans.labels_, return_counts = True)
+    freqs = np.array(freqs) / sum(freqs)
+    d_emb = (((1.0 - rho) * freqs + rho * (freqs ** 2)) ** 2).sum()
+    d_neo = 1.0 - d_emb
 
     # standardize and calculate PCA for sim_rawX
     simX = sim_rawX.astype(np.float32).toarray()
@@ -375,49 +378,53 @@ def _run_scrublet(
     assert maxima.size > 0
     curv = _calc_vec_f(_curvature, x.size, y, gap) # calculate curvature
 
+    x_theory = np.percentile(sim_scores_log, d_emb * 100.0 + 1e-6)
+    threshold_theory = np.exp(x_theory)
+
+    case_num = -1
+    pos = -1
     if maxima.size >= 2:
-        pos = _locate_cutoff_among_peaks(y, maxima)
-    else:
-        frac_right_thre = 0.41
-        frac_left_thre = 0.39
-
-        pos = -1
-        for i in range(maxima_by_x.size):
-            frac_right = (sim_scores_log > x[maxima_by_x[i]]).sum() / sim_scores.size
-            if frac_right < frac_right_thre: # peak might represent a doublet peak, try to find a cutoff at the left side
-                if i == 0:
-                    peak_curv_value = _find_curv_minima_at_peak(curv, maxima_by_x[i])
-                    end = _find_pos_curv(curv, maxima_by_x[i]-1, '-')
-                    start = _find_pos_curv(curv, _find_curv_local_minima(curv, peak_curv_value, filtered_maxima, end-1, '-')+1, '+')
-                    assert start <= end
-                    pos = curv[start:end+1].argmax() + start
-                else:
-                    pos = y[maxima_by_x[i-1]+1:maxima_by_x[i]].argmin() + (maxima_by_x[i-1]+1)
-
-                frac_left = (sim_scores_log < x[pos]).sum() / sim_scores.size    
-                if frac_left < frac_left_thre:
-                    pos = maxima_by_x[i]
-
-                break
-
-        if pos < 0:
-            # peak represents embedded doublets, find a cutoff at the right side
-            peak_curv_value = _find_curv_minima_at_peak(curv, maxima_by_x[-1])
-            start = _find_pos_curv(curv, maxima_by_x[-1]+1, '+')
-            end = _find_pos_curv(curv, _find_curv_local_minima(curv, peak_curv_value, filtered_maxima, start+1, '+')-1, '-')
-            assert start <= end
-            pos = curv[start:end+1].argmax() + start
-
+        pos = _locate_cutoff_among_peaks_with_guide(x, y, maxima, sim_scores_log, d_neo)
+        case_num = 0
+        d_pneo = (sim_scores_log > x[pos]).sum() / sim_scores_log.size
+        if d_pneo < 0.1: # < 10%, consider it as not a peak
+            idx_ = maxima_by_x >= pos
+            filtered_maxima = np.concatenate((filtered_maxima, maxima_by_x[idx_]))
+            maxima_by_x = maxima_by_x[~idx_]
+            pos = -1
+    if pos < 0:
+        frac_right = (sim_scores_log > x[maxima_by_x[-1]]).sum() / sim_scores.size
+        if frac_right < 0.41 or (frac_right < 0.5 and x_theory + 0.05 < x[maxima_by_x[-1]]):
+            logger.debug(f"frac_right={frac_right}.")
+            if maxima_by_x.size > 1:
+                posvec = np.vectorize(lambda i: y[maxima_by_x[i]+1:maxima_by_x[i+1]].argmin() + (maxima_by_x[i]+1))(range(maxima_by_x.size-1))
+                pos = posvec[np.argmin(np.abs(x[posvec] - x_theory))]
+                case_num = 1
+            else:
+                pos = _find_cutoff_left_side(maxima_by_x[0], x, curv, x_theory)
+                case_num = 2
+        else:
+            pos = _find_cutoff_right_side(maxima_by_x[-1], curv, filtered_maxima)
+            case_num = 3
     threshold = np.exp(x[pos])
 
+    threshold_auto = None
+    if manual_correction is not None:
+        assert case_num == 2
+        threshold_auto = threshold
+        threshold = np.exp(x[maxima_by_x[-1]])
+
     data.obs["doublet_score"] = obs_scores.astype(np.float32)
     data.obs["pred_dbl"] = obs_scores > threshold
     data.uns["doublet_threshold"] = float(threshold)
 
-    logger.info(f"Sample {name}: doublet threshold = {threshold:.4f}; total cells = {data.shape[0]}; neotypic doublet rate = {data.obs['pred_dbl'].sum() / data.shape[0]:.2%}")
+    neo_dbl_rate = data.obs['pred_dbl'].sum() / data.shape[0]
+    neo_sim_dbl_rate = (sim_scores > threshold).sum() / sim_scores.size
+    logger.info(f"Sample {name}: doublet threshold = {threshold:.4f}; total cells = {data.shape[0]}; neotypic doublet rate in simulation = {neo_sim_dbl_rate:.2%}; neotypic doublet rate = {neo_dbl_rate:.2%}.")
+
     fig = None
     if plot_hist:
-        fig = _plot_hist(obs_scores, sim_scores, threshold, x, y, curv)
+        fig = _plot_hist(obs_scores, sim_scores, threshold, threshold_theory, x, y, curv, threshold_auto=threshold_auto)
     return fig
 
 
@@ -469,6 +476,7 @@ def infer_doublets(
     alpha: Optional[float] = 0.05,
     random_state: Optional[int] = 0,
     plot_hist: Optional[str] = "sample",
+    manual_correction: Optional[str] = None,
 ) -> None:
     """Infer doublets by first calculating Scrublet-like [Wolock18]_ doublet scores and then smartly determining an appropriate doublet score cutoff [Li20-2]_ .
 
@@ -511,6 +519,9 @@ def infer_doublets(
 
     plot_hist: ``str``, optional, default: ``sample``
         If not None, plot diagnostic histograms using ``plot_hist`` as the prefix. If `channel_attr` is None, ``plot_hist.dbl.png`` is generated; Otherwise, ``plot_hist.channel_name.dbl.png`` files are generated. Each figure consists of 4 panels showing histograms of doublet scores for observed cells (panel 1, density in log scale), simulated doublets (panel 2, density in log scale), KDE plot (panel 3) and signed curvature plot (panel 4) of log doublet scores for simulated doublets.
+    
+    manual_correction: ``str``, optional, default: ``None``
+        Use human guide to correct doublet threshold for certain channels. This is string representing a comma-separately list. Each item in the list represent one sample and the sample name and correction guide are separated using ':'. The only correction guide supported is 'peak', which means cut at the center of the peak. If only one sample available, use '' as the sample name.
 
     Returns
     -------
@@ -533,10 +544,16 @@ def infer_doublets(
 
     if_plot = plot_hist is not None
 
+    mancor = {}
+    if manual_correction is not None:
+        for item in manual_correction.split(','):
+            name, action = item.split(':')
+            mancor[name] = action
+
     if channel_attr is None:
         if data.shape[0] >= min_cell:
             fig = _run_scrublet(data, expected_doublet_rate = expected_doublet_rate, sim_doublet_ratio = sim_doublet_ratio, \
-                                n_prin_comps = n_prin_comps, k = k, n_jobs = n_jobs, random_state = random_state, plot_hist = if_plot)
+                                n_prin_comps = n_prin_comps, k = k, n_jobs = n_jobs, random_state = random_state, plot_hist = if_plot, manual_correction = mancor.get('', None))
             if if_plot:
                 fig.savefig(f"{plot_hist}.dbl.png")
         else:
@@ -569,7 +586,7 @@ def infer_doublets(
                 highly_variable_features(unidata)
                 # Run _run_scrublet
                 fig = _run_scrublet(unidata, name = channel, expected_doublet_rate = expected_doublet_rate, sim_doublet_ratio = sim_doublet_ratio, \
-                                    n_prin_comps = n_prin_comps, k = k, n_jobs = n_jobs, random_state = random_state, plot_hist = if_plot)
+                                    n_prin_comps = n_prin_comps, k = k, n_jobs = n_jobs, random_state = random_state, plot_hist = if_plot, manual_correction = mancor.get(channel, None))
                 if if_plot:
                     fig.savefig(f"{plot_hist}.{channel}.dbl.png")
 
